package com.sun.org.apache.xml.internal.security.utils.resolver;

import java.util.ArrayList;
import com.sun.org.slf4j.internal.LoggerFactory;
import java.util.Map;
import com.sun.org.apache.xml.internal.security.signature.XMLSignatureInput;
import com.sun.org.apache.xml.internal.security.utils.resolver.implementations.ResolverXPointer;
import com.sun.org.apache.xml.internal.security.utils.resolver.implementations.ResolverFragment;
import com.sun.org.apache.xml.internal.security.utils.JavaUtils;
import java.util.Iterator;
import com.sun.org.apache.xml.internal.security.utils.resolver.implementations.ResolverDirectHTTP;
import com.sun.org.apache.xml.internal.security.utils.resolver.implementations.ResolverLocalFilesystem;
import org.w3c.dom.Attr;
import java.util.List;
import com.sun.org.slf4j.internal.Logger;

public class ResourceResolver
{
    private static final Logger LOG;
    private static final List<ResourceResolver> resolverList;
    private final ResourceResolverSpi resolverSpi;
    
    public ResourceResolver(ResourceResolverSpi resolverSpi) {
        this.resolverSpi = resolverSpi;
    }
    
    public static final ResourceResolver getInstance(Attr attr, String s, boolean b) throws ResourceResolverException {
        return internalGetInstance(new ResourceResolverContext(attr, s, b));
    }
    
    private static <N> ResourceResolver internalGetInstance(ResourceResolverContext resourceResolverContext) throws ResourceResolverException {
        synchronized (ResourceResolver.resolverList) {
            for (ResourceResolver resourceResolver : ResourceResolver.resolverList) {
                if (!resourceResolver.resolverSpi.engineIsThreadSafe()) {
                    try {
                        resourceResolver = new ResourceResolver((ResourceResolverSpi)resourceResolver.resolverSpi.getClass().newInstance());
                    }
                    catch (InstantiationException ex) {
                        throw new ResourceResolverException(ex, resourceResolverContext.uriToResolve, resourceResolverContext.baseUri, "");
                    }
                    catch (IllegalAccessException ex2) {
                        throw new ResourceResolverException(ex2, resourceResolverContext.uriToResolve, resourceResolverContext.baseUri, "");
                    }
                }
                ResourceResolver.LOG.debug("check resolvability by class {}", new Object[] { resourceResolver.getClass().getName() });
                if (resourceResolver.canResolve(resourceResolverContext)) {
                    if (resourceResolverContext.secureValidation && (resourceResolver.resolverSpi instanceof ResolverLocalFilesystem || resourceResolver.resolverSpi instanceof ResolverDirectHTTP)) {
                        throw new ResourceResolverException("signature.Reference.ForbiddenResolver", new Object[] { resourceResolver.resolverSpi.getClass().getName() }, resourceResolverContext.uriToResolve, resourceResolverContext.baseUri);
                    }
                    return resourceResolver;
                }
                else {
                    continue;
                }
            }
        }
        throw new ResourceResolverException("utils.resolver.noClass", new Object[] { (resourceResolverContext.uriToResolve != null) ? resourceResolverContext.uriToResolve : "null", resourceResolverContext.baseUri }, resourceResolverContext.uriToResolve, resourceResolverContext.baseUri);
    }
    
    public static ResourceResolver getInstance(Attr attr, String s, List<ResourceResolver> list) throws ResourceResolverException {
        return getInstance(attr, s, list, true);
    }
    
    public static ResourceResolver getInstance(Attr attr, String s, List<ResourceResolver> list, boolean b) throws ResourceResolverException {
        ResourceResolver.LOG.debug("I was asked to create a ResourceResolver and got {}", new Object[] { Integer.valueOf((list == null) ? 0 : list.size()) });
        ResourceResolverContext resourceResolverContext = new ResourceResolverContext(attr, s, b);
        if (list != null) {
            for (int i = 0; i < list.size(); ++i) {
                ResourceResolver resourceResolver = (ResourceResolver)list.get(i);
                if (resourceResolver != null) {
                    ResourceResolver.LOG.debug("check resolvability by class {}", new Object[] { resourceResolver.resolverSpi.getClass().getName() });
                    if (resourceResolver.canResolve(resourceResolverContext)) {
                        return resourceResolver;
                    }
                }
            }
        }
        return internalGetInstance(resourceResolverContext);
    }
    
    public static void register(String str) {
        JavaUtils.checkRegisterPermission();
        try {
            register(ClassLoaderUtils.loadClass(str, ResourceResolver.class), false);
        }
        catch (ClassNotFoundException ex) {
            ResourceResolver.LOG.warn("Error loading resolver " + str + " disabling it");
        }
    }
    
    public static void registerAtStart(String str) {
        JavaUtils.checkRegisterPermission();
        try {
            register(ClassLoaderUtils.loadClass(str, ResourceResolver.class), true);
        }
        catch (ClassNotFoundException ex) {
            ResourceResolver.LOG.warn("Error loading resolver " + str + " disabling it");
        }
    }
    
    public static void register(Class<? extends ResourceResolverSpi> clazz, boolean b) {
        JavaUtils.checkRegisterPermission();
        try {
            register((ResourceResolverSpi)clazz.newInstance(), b);
        }
        catch (IllegalAccessException ex) {
            ResourceResolver.LOG.warn("Error loading resolver " + clazz + " disabling it");
        }
        catch (InstantiationException ex2) {
            ResourceResolver.LOG.warn("Error loading resolver " + clazz + " disabling it");
        }
    }
    
    public static void register(ResourceResolverSpi resourceResolverSpi, boolean b) {
        JavaUtils.checkRegisterPermission();
        synchronized (ResourceResolver.resolverList) {
            if (b) {
                ResourceResolver.resolverList.add(0, new ResourceResolver(resourceResolverSpi));
            }
            else {
                ResourceResolver.resolverList.add(new ResourceResolver(resourceResolverSpi));
            }
        }
        ResourceResolver.LOG.debug("Registered resolver: {}", new Object[] { resourceResolverSpi.toString() });
    }
    
    public static void registerDefaultResolvers() {
        synchronized (ResourceResolver.resolverList) {
            ResourceResolver.resolverList.add(new ResourceResolver(new ResolverFragment()));
            ResourceResolver.resolverList.add(new ResourceResolver(new ResolverLocalFilesystem()));
            ResourceResolver.resolverList.add(new ResourceResolver(new ResolverXPointer()));
            ResourceResolver.resolverList.add(new ResourceResolver(new ResolverDirectHTTP()));
        }
    }
    
    public XMLSignatureInput resolve(Attr attr, String s, boolean b) throws ResourceResolverException {
        return this.resolverSpi.engineResolveURI(new ResourceResolverContext(attr, s, b));
    }
    
    public void setProperty(String s, String s2) {
        this.resolverSpi.engineSetProperty(s, s2);
    }
    
    public String getProperty(String s) {
        return this.resolverSpi.engineGetProperty(s);
    }
    
    public void addProperties(Map<String, String> map) {
        this.resolverSpi.engineAddProperies(map);
    }
    
    public String[] getPropertyKeys() {
        return this.resolverSpi.engineGetPropertyKeys();
    }
    
    public boolean understandsProperty(String s) {
        return this.resolverSpi.understandsProperty(s);
    }
    
    private boolean canResolve(ResourceResolverContext resourceResolverContext) {
        return this.resolverSpi.engineCanResolveURI(resourceResolverContext);
    }
    
    static {
        LOG = LoggerFactory.getLogger(ResourceResolver.class);
        resolverList = new ArrayList();
    }
}
